@@ -5,16 +5,16 @@ from django.utils import timezone
 class Migration(migrations.Migration):
 
     dependencies = [
-        ('admin', '0001_initial'),
+        ("admin", "0001_initial"),
     ]
 
     # No database changes; removes auto_add and adds default/editable.
     operations = [
         migrations.AlterField(
-            model_name='logentry',
-            name='action_time',
+            model_name="logentry",
+            name="action_time",
             field=models.DateTimeField(
-                verbose_name='action time',
+                verbose_name="action time",
                 default=timezone.now,
                 editable=False,
             ),
