@@ -12,7 +12,7 @@ class OracleIntrospection(DatabaseIntrospection):
     def data_types_reverse(self):
         return {
             **super().data_types_reverse,
-            cx_Oracle.OBJECT: 'GeometryField',
+            cx_Oracle.OBJECT: "GeometryField",
         }
 
     def get_geometry_type(self, table_name, description):
@@ -22,26 +22,26 @@ class OracleIntrospection(DatabaseIntrospection):
                 cursor.execute(
                     'SELECT "DIMINFO", "SRID" FROM "USER_SDO_GEOM_METADATA" '
                     'WHERE "TABLE_NAME"=%s AND "COLUMN_NAME"=%s',
-                    (table_name.upper(), description.name.upper())
+                    (table_name.upper(), description.name.upper()),
                 )
                 row = cursor.fetchone()
             except Exception as exc:
                 raise Exception(
-                    'Could not find entry in USER_SDO_GEOM_METADATA '
+                    "Could not find entry in USER_SDO_GEOM_METADATA "
                     'corresponding to "%s"."%s"' % (table_name, description.name)
                 ) from exc
 
             # TODO: Research way to find a more specific geometry field type for
             # the column's contents.
-            field_type = 'GeometryField'
+            field_type = "GeometryField"
 
             # Getting the field parameters.
             field_params = {}
             dim, srid = row
             if srid != 4326:
-                field_params['srid'] = srid
+                field_params["srid"] = srid
             # Size of object array (SDO_DIM_ARRAY) is number of dimensions.
             dim = dim.size()
             if dim != 2:
-                field_params['dim'] = dim
+                field_params["dim"] = dim
         return field_type, field_params
