@@ -5,7 +5,6 @@ be executed through ``django-admin`` or ``manage.py``).
 import argparse
 import os
 import sys
-import warnings
 from argparse import ArgumentParser, HelpFormatter
 from io import TextIOBase
 
@@ -14,7 +13,6 @@ from django.core import checks
 from django.core.exceptions import ImproperlyConfigured
 from django.core.management.color import color_style, no_style
 from django.db import DEFAULT_DB_ALIAS, connections
-from django.utils.deprecation import RemovedInDjango41Warning
 
 ALL_CHECKS = "__all__"
 
@@ -269,16 +267,6 @@ class BaseCommand:
         else:
             self.style = color_style(force_color)
             self.stderr.style_func = self.style.ERROR
-        if self.requires_system_checks in [False, True]:
-            warnings.warn(
-                "Using a boolean value for requires_system_checks is "
-                "deprecated. Use '__all__' instead of True, and [] (an empty "
-                "list) instead of False.",
-                RemovedInDjango41Warning,
-            )
-            self.requires_system_checks = (
-                ALL_CHECKS if self.requires_system_checks else []
-            )
         if (
             not isinstance(self.requires_system_checks, (list, tuple))
             and self.requires_system_checks != ALL_CHECKS
@@ -298,10 +286,10 @@ class BaseCommand:
         Create and return the ``ArgumentParser`` which will be used to
         parse the arguments to this command.
         """
+        kwargs.setdefault("formatter_class", DjangoHelpFormatter)
         parser = CommandParser(
             prog="%s %s" % (os.path.basename(prog_name), subcommand),
             description=self.help or None,
-            formatter_class=DjangoHelpFormatter,
             missing_args_message=getattr(self, "missing_args_message", None),
             called_from_command_line=getattr(self, "_called_from_command_line", None),
             **kwargs,
