@@ -23,6 +23,7 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
     sql_delete_fk = "ALTER TABLE %(table)s DROP FOREIGN KEY %(name)s"
 
     sql_delete_index = "DROP INDEX %(name)s ON %(table)s"
+    sql_rename_index = "ALTER TABLE %(table)s RENAME INDEX %(old_name)s TO %(new_name)s"
 
     sql_create_pk = (
         "ALTER TABLE %(table)s ADD CONSTRAINT %(name)s PRIMARY KEY (%(columns)s)"
@@ -84,10 +85,9 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
 
     @property
     def _supports_limited_data_type_defaults(self):
-        # MariaDB >= 10.2.1 and MySQL >= 8.0.13 supports defaults for BLOB
-        # and TEXT.
+        # MariaDB and MySQL >= 8.0.13 support defaults for BLOB and TEXT.
         if self.connection.mysql_is_mariadb:
-            return self.connection.mysql_version >= (10, 2, 1)
+            return True
         return self.connection.mysql_version >= (8, 0, 13)
 
     def _column_default_sql(self, field):
@@ -140,7 +140,7 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         MySQL can remove an implicit FK index on a field when that field is
         covered by another index like a unique_together. "covered" here means
         that the more complex index starts like the simpler one.
-        http://bugs.mysql.com/bug.php?id=37910 / Django ticket #24757
+        https://bugs.mysql.com/bug.php?id=37910 / Django ticket #24757
         We check here before removing the [unique|index]_together if we have to
         recreate a FK index.
         """
