@@ -1,5 +1,4 @@
 import operator
-import platform
 
 from django.db import transaction
 from django.db.backends.base.features import BaseDatabaseFeatures
@@ -10,28 +9,28 @@ from .base import Database
 
 
 class DatabaseFeatures(BaseDatabaseFeatures):
-    # SQLite can read from a cursor since SQLite 3.6.5, subject to the caveat
-    # that statements within a connection aren't isolated from each other. See
-    # https://sqlite.org/isolation.html.
-    can_use_chunked_reads = True
+    minimum_database_version = (3, 9)
     test_db_allows_multiple_connections = False
     supports_unspecified_pk = True
     supports_timezones = False
     max_query_params = 999
-    supports_mixed_date_datetime_comparisons = False
     supports_transactions = True
     atomic_transactions = False
     can_rollback_ddl = True
     can_create_inline_fk = False
     supports_paramstyle_pyformat = False
+    requires_literal_defaults = True
     can_clone_databases = True
     supports_temporal_subtraction = True
     ignores_table_name_case = True
     supports_cast_with_precision = False
     time_cast_precision = 3
     can_release_savepoints = True
+    has_case_insensitive_like = True
     # Is "ALTER TABLE ... RENAME COLUMN" supported?
     can_alter_table_rename_column = Database.sqlite_version_info >= (3, 25, 0)
+    # Is "ALTER TABLE ... DROP COLUMN" supported?
+    can_alter_table_drop_column = Database.sqlite_version_info >= (3, 35, 5)
     supports_parentheses_in_compound = False
     # Deferred constraint checks can be emulated on SQLite < 3.20 but not in a
     # reasonably performant way.
@@ -44,11 +43,18 @@ class DatabaseFeatures(BaseDatabaseFeatures):
     supports_order_by_nulls_modifier = Database.sqlite_version_info >= (3, 30, 0)
     order_by_nulls_first = True
     supports_json_field_contains = False
+    supports_update_conflicts = Database.sqlite_version_info >= (3, 24, 0)
+    supports_update_conflicts_with_target = supports_update_conflicts
     test_collations = {
         "ci": "nocase",
         "cs": "binary",
         "non_default": "nocase",
     }
+    django_test_expected_failures = {
+        # The django_format_dtdelta() function doesn't properly handle mixed
+        # Date/DateTime fields and timedeltas.
+        "expressions.tests.FTimeDeltaTests.test_mixed_comparisons1",
+    }
 
     @cached_property
     def django_test_skips(self):
@@ -65,10 +71,6 @@ class DatabaseFeatures(BaseDatabaseFeatures):
                 "schema.tests.SchemaTests."
                 "test_rename_column_renames_deferred_sql_references",
             },
-            "SQLite doesn't have a constraint.": {
-                "model_fields.test_integerfield.PositiveIntegerFieldTests."
-                "test_negative_values",
-            },
             "SQLite doesn't support negative precision for ROUND().": {
                 "db_functions.math.test_round.RoundTests."
                 "test_null_with_negative_precision",
@@ -105,12 +107,6 @@ class DatabaseFeatures(BaseDatabaseFeatures):
 
     @cached_property
     def supports_atomic_references_rename(self):
-        # SQLite 3.28.0 bundled with MacOS 10.15 does not support renaming
-        # references atomically.
-        if platform.mac_ver()[0].startswith(
-            "10.15."
-        ) and Database.sqlite_version_info == (3, 28, 0):
-            return False
         return Database.sqlite_version_info >= (3, 26, 0)
 
     @cached_property
