@@ -1,5 +1,6 @@
 import functools
 import re
+from collections import defaultdict
 from itertools import chain
 
 from django.conf import settings
@@ -96,7 +97,7 @@ class MigrationAutodetector:
         for name, field in sorted(fields.items()):
             deconstruction = self.deep_deconstruct(field)
             if field.remote_field and field.remote_field.model:
-                del deconstruction[2]["to"]
+                deconstruction[2].pop("to", None)
             fields_def.append(deconstruction)
         return fields_def
 
@@ -122,6 +123,7 @@ class MigrationAutodetector:
         self.generated_operations = {}
         self.altered_indexes = {}
         self.altered_constraints = {}
+        self.renamed_fields = {}
 
         # Prepare some old/new state and model lists, separating
         # proxy models and ignoring unmigrated apps.
@@ -169,6 +171,11 @@ class MigrationAutodetector:
         self.generate_altered_options()
         self.generate_altered_managers()
 
+        # Create the renamed fields and store them in self.renamed_fields.
+        # They are used by create_altered_indexes(), generate_altered_fields(),
+        # generate_removed_altered_index/unique_together(), and
+        # generate_altered_index/unique_together().
+        self.create_renamed_fields()
         # Create the altered indexes and store them in self.altered_indexes.
         # This avoids the same computation in generate_removed_indexes()
         # and generate_added_indexes().
@@ -179,6 +186,7 @@ class MigrationAutodetector:
         self.generate_removed_indexes()
         # Generate field renaming operations.
         self.generate_renamed_fields()
+        self.generate_renamed_indexes()
         # Generate removal of foo together.
         self.generate_removed_altered_unique_together()
         self.generate_removed_altered_index_together()
@@ -906,11 +914,12 @@ class MigrationAutodetector:
                 ),
             )
 
-    def generate_renamed_fields(self):
+    def create_renamed_fields(self):
         """Work out renamed fields."""
-        self.renamed_fields = {}
+        self.renamed_operations = []
+        old_field_keys = self.old_field_keys.copy()
         for app_label, model_name, field_name in sorted(
-            self.new_field_keys - self.old_field_keys
+            self.new_field_keys - old_field_keys
         ):
             old_model_name = self.renamed_models.get(
                 (app_label, model_name), model_name
@@ -921,7 +930,7 @@ class MigrationAutodetector:
             # Scan to see if this is actually a rename!
             field_dec = self.deep_deconstruct(field)
             for rem_app_label, rem_model_name, rem_field_name in sorted(
-                self.old_field_keys - self.new_field_keys
+                old_field_keys - self.new_field_keys
             ):
                 if rem_app_label == app_label and rem_model_name == model_name:
                     old_field = old_model_state.get_field(rem_field_name)
@@ -946,23 +955,64 @@ class MigrationAutodetector:
                         if self.questioner.ask_rename(
                             model_name, rem_field_name, field_name, field
                         ):
-                            self.add_operation(
-                                app_label,
-                                operations.RenameField(
-                                    model_name=model_name,
-                                    old_name=rem_field_name,
-                                    new_name=field_name,
-                                ),
+                            self.renamed_operations.append(
+                                (
+                                    rem_app_label,
+                                    rem_model_name,
+                                    old_field.db_column,
+                                    rem_field_name,
+                                    app_label,
+                                    model_name,
+                                    field,
+                                    field_name,
+                                )
                             )
-                            self.old_field_keys.remove(
+                            old_field_keys.remove(
                                 (rem_app_label, rem_model_name, rem_field_name)
                             )
-                            self.old_field_keys.add((app_label, model_name, field_name))
+                            old_field_keys.add((app_label, model_name, field_name))
                             self.renamed_fields[
                                 app_label, model_name, field_name
                             ] = rem_field_name
                             break
 
+    def generate_renamed_fields(self):
+        """Generate RenameField operations."""
+        for (
+            rem_app_label,
+            rem_model_name,
+            rem_db_column,
+            rem_field_name,
+            app_label,
+            model_name,
+            field,
+            field_name,
+        ) in self.renamed_operations:
+            # A db_column mismatch requires a prior noop AlterField for the
+            # subsequent RenameField to be a noop on attempts at preserving the
+            # old name.
+            if rem_db_column != field.db_column:
+                altered_field = field.clone()
+                altered_field.name = rem_field_name
+                self.add_operation(
+                    app_label,
+                    operations.AlterField(
+                        model_name=model_name,
+                        name=rem_field_name,
+                        field=altered_field,
+                    ),
+                )
+            self.add_operation(
+                app_label,
+                operations.RenameField(
+                    model_name=model_name,
+                    old_name=rem_field_name,
+                    new_name=field_name,
+                ),
+            )
+            self.old_field_keys.remove((rem_app_label, rem_model_name, rem_field_name))
+            self.old_field_keys.add((app_label, model_name, field_name))
+
     def generate_added_fields(self):
         """Make AddField operations."""
         for app_label, model_name, field_name in sorted(
@@ -972,8 +1022,9 @@ class MigrationAutodetector:
 
     def _generate_added_field(self, app_label, model_name, field_name):
         field = self.to_state.models[app_label, model_name].get_field(field_name)
-        # Fields that are foreignkeys/m2ms depend on stuff
-        dependencies = []
+        # Adding a field always depends at least on its removal.
+        dependencies = [(app_label, model_name, field_name, False)]
+        # Fields that are foreignkeys/m2ms depend on stuff.
         if field.remote_field and field.remote_field.model:
             dependencies.extend(
                 self._get_dependencies_for_foreign_key(
@@ -1003,6 +1054,12 @@ class MigrationAutodetector:
                 field.default = self.questioner.ask_not_null_addition(
                     field_name, model_name
                 )
+        if (
+            field.unique
+            and field.default is not models.NOT_PROVIDED
+            and callable(field.default)
+        ):
+            self.questioner.ask_unique_callable_default_addition(field_name, model_name)
         self.add_operation(
             app_label,
             operations.AddField(
@@ -1117,7 +1174,10 @@ class MigrationAutodetector:
                     new_field.remote_field.through = old_field.remote_field.through
             old_field_dec = self.deep_deconstruct(old_field)
             new_field_dec = self.deep_deconstruct(new_field)
-            if old_field_dec != new_field_dec:
+            # If the field was confirmed to be renamed it means that only
+            # db_column was allowed to change which generate_renamed_fields()
+            # already accounts for by adding an AlterField operation.
+            if old_field_dec != new_field_dec and old_field_name == field_name:
                 both_m2m = old_field.many_to_many and new_field.many_to_many
                 neither_m2m = not old_field.many_to_many and not new_field.many_to_many
                 if both_m2m or neither_m2m:
@@ -1155,6 +1215,8 @@ class MigrationAutodetector:
 
     def create_altered_indexes(self):
         option_name = operations.AddIndex.option_name
+        self.renamed_index_together_values = defaultdict(list)
+
         for app_label, model_name in sorted(self.kept_model_keys):
             old_model_name = self.renamed_models.get(
                 (app_label, model_name), model_name
@@ -1164,14 +1226,78 @@ class MigrationAutodetector:
 
             old_indexes = old_model_state.options[option_name]
             new_indexes = new_model_state.options[option_name]
-            add_idx = [idx for idx in new_indexes if idx not in old_indexes]
-            rem_idx = [idx for idx in old_indexes if idx not in new_indexes]
+            added_indexes = [idx for idx in new_indexes if idx not in old_indexes]
+            removed_indexes = [idx for idx in old_indexes if idx not in new_indexes]
+            renamed_indexes = []
+            # Find renamed indexes.
+            remove_from_added = []
+            remove_from_removed = []
+            for new_index in added_indexes:
+                new_index_dec = new_index.deconstruct()
+                new_index_name = new_index_dec[2].pop("name")
+                for old_index in removed_indexes:
+                    old_index_dec = old_index.deconstruct()
+                    old_index_name = old_index_dec[2].pop("name")
+                    # Indexes are the same except for the names.
+                    if (
+                        new_index_dec == old_index_dec
+                        and new_index_name != old_index_name
+                    ):
+                        renamed_indexes.append((old_index_name, new_index_name, None))
+                        remove_from_added.append(new_index)
+                        remove_from_removed.append(old_index)
+            # Find index_together changed to indexes.
+            for (
+                old_value,
+                new_value,
+                index_together_app_label,
+                index_together_model_name,
+                dependencies,
+            ) in self._get_altered_foo_together_operations(
+                operations.AlterIndexTogether.option_name
+            ):
+                if (
+                    app_label != index_together_app_label
+                    or model_name != index_together_model_name
+                ):
+                    continue
+                removed_values = old_value.difference(new_value)
+                for removed_index_together in removed_values:
+                    renamed_index_together_indexes = []
+                    for new_index in added_indexes:
+                        _, args, kwargs = new_index.deconstruct()
+                        # Ensure only 'fields' are defined in the Index.
+                        if (
+                            not args
+                            and new_index.fields == list(removed_index_together)
+                            and set(kwargs) == {"name", "fields"}
+                        ):
+                            renamed_index_together_indexes.append(new_index)
+
+                    if len(renamed_index_together_indexes) == 1:
+                        renamed_index = renamed_index_together_indexes[0]
+                        remove_from_added.append(renamed_index)
+                        renamed_indexes.append(
+                            (None, renamed_index.name, removed_index_together)
+                        )
+                        self.renamed_index_together_values[
+                            index_together_app_label, index_together_model_name
+                        ].append(removed_index_together)
+            # Remove renamed indexes from the lists of added and removed
+            # indexes.
+            added_indexes = [
+                idx for idx in added_indexes if idx not in remove_from_added
+            ]
+            removed_indexes = [
+                idx for idx in removed_indexes if idx not in remove_from_removed
+            ]
 
             self.altered_indexes.update(
                 {
                     (app_label, model_name): {
-                        "added_indexes": add_idx,
-                        "removed_indexes": rem_idx,
+                        "added_indexes": added_indexes,
+                        "removed_indexes": removed_indexes,
+                        "renamed_indexes": renamed_indexes,
                     }
                 }
             )
@@ -1198,6 +1324,21 @@ class MigrationAutodetector:
                     ),
                 )
 
+    def generate_renamed_indexes(self):
+        for (app_label, model_name), alt_indexes in self.altered_indexes.items():
+            for old_index_name, new_index_name, old_fields in alt_indexes[
+                "renamed_indexes"
+            ]:
+                self.add_operation(
+                    app_label,
+                    operations.RenameIndex(
+                        model_name=model_name,
+                        new_name=new_index_name,
+                        old_name=old_index_name,
+                        old_fields=old_fields,
+                    ),
+                )
+
     def create_altered_constraints(self):
         option_name = operations.AddConstraint.option_name
         for app_label, model_name in sorted(self.kept_model_keys):
@@ -1277,7 +1418,7 @@ class MigrationAutodetector:
         dependencies = [(dep_app_label, dep_object_name, None, True)]
         if getattr(field.remote_field, "through", None):
             through_app_label, through_object_name = resolve_relation(
-                remote_field_model,
+                field.remote_field.through,
                 app_label,
                 model_name,
             )
@@ -1339,6 +1480,13 @@ class MigrationAutodetector:
             model_name,
             dependencies,
         ) in self._get_altered_foo_together_operations(operation.option_name):
+            if operation == operations.AlterIndexTogether:
+                old_value = {
+                    value
+                    for value in old_value
+                    if value
+                    not in self.renamed_index_together_values[app_label, model_name]
+                }
             removal_value = new_value.intersection(old_value)
             if removal_value or old_value:
                 self.add_operation(
@@ -1571,8 +1719,11 @@ class MigrationAutodetector:
     def parse_number(cls, name):
         """
         Given a migration name, try to extract a number from the beginning of
-        it. If no number is found, return None.
+        it. For a squashed migration such as '0001_squashed_0004…', return the
+        second number. If no number is found, return None.
         """
+        if squashed_match := re.search(r".*_squashed_(\d+)", name):
+            return int(squashed_match[1])
         match = re.match(r"^\d+", name)
         if match:
             return int(match[0])
