@@ -130,9 +130,9 @@ class Field(RegisterLookupMixin):
         "null": _("This field cannot be null."),
         "blank": _("This field cannot be blank."),
         "unique": _("%(model_name)s with this %(field_label)s already exists."),
-        # Translators: The 'lookup_type' is one of 'date', 'year' or 'month'.
-        # Eg: "Title must be unique for pub_date year"
         "unique_for_date": _(
+            # Translators: The 'lookup_type' is one of 'date', 'year' or
+            # 'month'. Eg: "Title must be unique for pub_date year"
             "%(field_label)s must be unique for "
             "%(date_field_label)s %(lookup_type)s."
         ),
@@ -140,6 +140,24 @@ class Field(RegisterLookupMixin):
     system_check_deprecated_details = None
     system_check_removed_details = None
 
+    # Attributes that don't affect a column definition.
+    # These attributes are ignored when altering the field.
+    non_db_attrs = (
+        "blank",
+        "choices",
+        "db_column",
+        "editable",
+        "error_messages",
+        "help_text",
+        "limit_choices_to",
+        # Database-level options are not supported, see #21961.
+        "on_delete",
+        "related_name",
+        "related_query_name",
+        "validators",
+        "verbose_name",
+    )
+
     # Field flags
     hidden = False
 
@@ -217,12 +235,7 @@ class Field(RegisterLookupMixin):
 
         self._validators = list(validators)  # Store for deconstruction later
 
-        messages = {}
-        for c in reversed(self.__class__.__mro__):
-            messages.update(getattr(c, "default_error_messages", {}))
-        messages.update(error_messages or {})
         self._error_messages = error_messages  # Store for deconstruction later
-        self.error_messages = messages
 
     def __str__(self):
         """
@@ -606,13 +619,7 @@ class Field(RegisterLookupMixin):
         return NotImplemented
 
     def __hash__(self):
-        return hash(
-            (
-                self.creation_counter,
-                self.model._meta.app_label if hasattr(self, "model") else None,
-                self.model._meta.model_name if hasattr(self, "model") else None,
-            )
-        )
+        return hash(self.creation_counter)
 
     def __deepcopy__(self, memodict):
         # We don't have to deepcopy very much here, since most things are not
@@ -675,6 +682,14 @@ class Field(RegisterLookupMixin):
         """
         return value
 
+    @cached_property
+    def error_messages(self):
+        messages = {}
+        for c in reversed(self.__class__.__mro__):
+            messages.update(getattr(c, "default_error_messages", {}))
+        messages.update(self._error_messages or {})
+        return messages
+
     @cached_property
     def validators(self):
         """
@@ -1089,6 +1104,15 @@ class BooleanField(Field):
             defaults = {"form_class": form_class, "required": False}
         return super().formfield(**{**defaults, **kwargs})
 
+    def select_format(self, compiler, sql, params):
+        sql, params = super().select_format(compiler, sql, params)
+        # Filters that match everything are handled as empty strings in the
+        # WHERE clause, but in SELECT or GROUP BY list they must use a
+        # predicate that's always True.
+        if sql == "":
+            sql = "1"
+        return sql, params
+
 
 class CharField(Field):
     description = _("String (up to %(max_length)s)")
@@ -1096,7 +1120,8 @@ class CharField(Field):
     def __init__(self, *args, db_collation=None, **kwargs):
         super().__init__(*args, **kwargs)
         self.db_collation = db_collation
-        self.validators.append(validators.MaxLengthValidator(self.max_length))
+        if self.max_length is not None:
+            self.validators.append(validators.MaxLengthValidator(self.max_length))
 
     def check(self, **kwargs):
         databases = kwargs.get("databases") or []
@@ -1157,6 +1182,11 @@ class CharField(Field):
             return connection.ops.cast_char_field_without_max_length
         return super().cast_db_type(connection)
 
+    def db_parameters(self, connection):
+        db_params = super().db_parameters(connection)
+        db_params["collation"] = self.db_collation
+        return db_params
+
     def get_internal_type(self):
         return "CharField"
 
@@ -1205,7 +1235,7 @@ class CommaSeparatedIntegerField(CharField):
 
 def _to_naive(value):
     if timezone.is_aware(value):
-        value = timezone.make_naive(value, timezone.utc)
+        value = timezone.make_naive(value, datetime.timezone.utc)
     return value
 
 
@@ -2336,6 +2366,11 @@ class TextField(Field):
                 )
         return errors
 
+    def db_parameters(self, connection):
+        db_params = super().db_parameters(connection)
+        db_params["collation"] = self.db_collation
+        return db_params
+
     def get_internal_type(self):
         return "TextField"
 
