@@ -153,7 +153,7 @@ class BoundField:
             initial_value = self.initial
         return field.has_changed(initial_value, self.data)
 
-    def label_tag(self, contents=None, attrs=None, label_suffix=None):
+    def label_tag(self, contents=None, attrs=None, label_suffix=None, tag=None):
         """
         Wrap the given contents in a <label>, if the field has an ID attribute.
         contents should be mark_safe'd to avoid HTML escaping. If contents
@@ -192,9 +192,22 @@ class BoundField:
             "label": contents,
             "attrs": attrs,
             "use_tag": bool(id_),
+            "tag": tag or "label",
         }
         return self.form.render(self.form.template_name_label, context)
 
+    def legend_tag(self, contents=None, attrs=None, label_suffix=None):
+        """
+        Wrap the given contents in a <legend>, if the field has an ID
+        attribute. Contents should be mark_safe'd to avoid HTML escaping. If
+        contents aren't given, use the field's HTML-escaped label.
+
+        If attrs are given, use them as HTML attributes on the <legend> tag.
+
+        label_suffix overrides the form's label_suffix.
+        """
+        return self.label_tag(contents, attrs, label_suffix, tag="legend")
+
     def css_classes(self, extra_classes=None):
         """
         Return a string of space-separated CSS classes for this field.
@@ -273,6 +286,13 @@ class BoundField:
             r"widget$|input$", "", self.field.widget.__class__.__name__.lower()
         )
 
+    @property
+    def use_fieldset(self):
+        """
+        Return the value of this BoundField widget's use_fieldset attribute.
+        """
+        return self.field.widget.use_fieldset
+
 
 @html_safe
 class BoundWidget:
