@@ -1,5 +1,6 @@
 import asyncio
 import difflib
+import inspect
 import json
 import logging
 import posixpath
@@ -37,13 +38,15 @@ from django.core.management import call_command
 from django.core.management.color import no_style
 from django.core.management.sql import emit_post_migrate_signal
 from django.core.servers.basehttp import ThreadedWSGIServer, WSGIRequestHandler
+from django.core.signals import setting_changed
 from django.db import DEFAULT_DB_ALIAS, connection, connections, transaction
 from django.forms.fields import CharField
 from django.http import QueryDict
 from django.http.request import split_domain_port, validate_host
+from django.http.response import HttpResponseBase
 from django.test.client import AsyncClient, Client
 from django.test.html import HTMLParseError, parse_html
-from django.test.signals import setting_changed, template_rendered
+from django.test.signals import template_rendered
 from django.test.utils import (
     CaptureQueriesContext,
     ContextList,
@@ -51,7 +54,7 @@ from django.test.utils import (
     modify_settings,
     override_settings,
 )
-from django.utils.deprecation import RemovedInDjango41Warning
+from django.utils.deprecation import RemovedInDjango50Warning
 from django.utils.functional import classproperty
 from django.utils.version import PY310
 from django.views.static import serve
@@ -66,13 +69,8 @@ __all__ = (
 
 
 def to_list(value):
-    """
-    Put value into a list if it's not already one. Return an empty list if
-    value is None.
-    """
-    if value is None:
-        value = []
-    elif not isinstance(value, list):
+    """Put value into a list if it's not already one."""
+    if not isinstance(value, list):
         value = [value]
     return value
 
@@ -113,9 +111,12 @@ class _AssertNumQueriesContext(CaptureQueriesContext):
 
 
 class _AssertTemplateUsedContext:
-    def __init__(self, test_case, template_name):
+    def __init__(self, test_case, template_name, msg_prefix="", count=None):
         self.test_case = test_case
         self.template_name = template_name
+        self.msg_prefix = msg_prefix
+        self.count = count
+
         self.rendered_templates = []
         self.rendered_template_names = []
         self.context = ContextList()
@@ -126,10 +127,12 @@ class _AssertTemplateUsedContext:
         self.context.append(copy(context))
 
     def test(self):
-        return self.template_name in self.rendered_template_names
-
-    def message(self):
-        return "%s was not rendered." % self.template_name
+        self.test_case._assert_template_used(
+            self.template_name,
+            self.rendered_template_names,
+            self.msg_prefix,
+            self.count,
+        )
 
     def __enter__(self):
         template_rendered.connect(self.on_template_render)
@@ -139,24 +142,20 @@ class _AssertTemplateUsedContext:
         template_rendered.disconnect(self.on_template_render)
         if exc_type is not None:
             return
-
-        if not self.test():
-            message = self.message()
-            if self.rendered_templates:
-                message += " Following templates were rendered: %s" % (
-                    ", ".join(self.rendered_template_names)
-                )
-            else:
-                message += " No template was rendered."
-            self.test_case.fail(message)
+        self.test()
 
 
 class _AssertTemplateNotUsedContext(_AssertTemplateUsedContext):
     def test(self):
-        return self.template_name not in self.rendered_template_names
+        self.test_case.assertFalse(
+            self.template_name in self.rendered_template_names,
+            f"{self.msg_prefix}Template '{self.template_name}' was used "
+            f"unexpectedly in rendering the response",
+        )
 
-    def message(self):
-        return "%s was rendered." % self.template_name
+
+class DatabaseOperationForbidden(AssertionError):
+    pass
 
 
 class _DatabaseFailure:
@@ -165,7 +164,128 @@ class _DatabaseFailure:
         self.message = message
 
     def __call__(self):
-        raise AssertionError(self.message)
+        raise DatabaseOperationForbidden(self.message)
+
+
+# RemovedInDjango50Warning
+class _AssertFormErrorDeprecationHelper:
+    @staticmethod
+    def assertFormError(self, response, form, field, errors, msg_prefix=""):
+        """
+        Search through all the rendered contexts of the `response` for a form named
+        `form` then dispatch to the new assertFormError() using that instance.
+        If multiple contexts contain the form, they're all checked in order and any
+        failure will abort (this matches the old behavior).
+        """
+        warning_msg = (
+            f"Passing response to assertFormError() is deprecated. Use the form object "
+            f"directly: assertFormError(response.context[{form!r}], {field!r}, ...)"
+        )
+        warnings.warn(warning_msg, RemovedInDjango50Warning, stacklevel=2)
+
+        full_msg_prefix = f"{msg_prefix}: " if msg_prefix else ""
+        contexts = to_list(response.context) if response.context is not None else []
+        if not contexts:
+            self.fail(
+                f"{full_msg_prefix}Response did not use any contexts to render the "
+                f"response"
+            )
+        # Search all contexts for the error.
+        found_form = False
+        for i, context in enumerate(contexts):
+            if form not in context:
+                continue
+            found_form = True
+            self.assertFormError(context[form], field, errors, msg_prefix=msg_prefix)
+        if not found_form:
+            self.fail(
+                f"{full_msg_prefix}The form '{form}' was not used to render the "
+                f"response"
+            )
+
+    @staticmethod
+    def assertFormsetError(
+        self, response, formset, form_index, field, errors, msg_prefix=""
+    ):
+        """
+        Search for a formset named "formset" in the "response" and dispatch to
+        the new assertFormsetError() using that instance. If the name is found
+        in multiple contexts they're all checked in order and any failure will
+        abort the test.
+        """
+        warning_msg = (
+            f"Passing response to assertFormsetError() is deprecated. Use the formset "
+            f"object directly: assertFormsetError(response.context[{formset!r}], "
+            f"{form_index!r}, ...)"
+        )
+        warnings.warn(warning_msg, RemovedInDjango50Warning, stacklevel=2)
+
+        full_msg_prefix = f"{msg_prefix}: " if msg_prefix else ""
+        contexts = to_list(response.context) if response.context is not None else []
+        if not contexts:
+            self.fail(
+                f"{full_msg_prefix}Response did not use any contexts to render the "
+                f"response"
+            )
+        found_formset = False
+        for i, context in enumerate(contexts):
+            if formset not in context or not hasattr(context[formset], "forms"):
+                continue
+            found_formset = True
+            self.assertFormsetError(
+                context[formset], form_index, field, errors, msg_prefix
+            )
+        if not found_formset:
+            self.fail(
+                f"{full_msg_prefix}The formset '{formset}' was not used to render the "
+                f"response"
+            )
+
+    @classmethod
+    def patch_signature(cls, new_method):
+        """
+        Replace the decorated method with a new one that inspects the passed
+        args/kwargs and dispatch to the old implementation (with deprecation
+        warning) when it detects the old signature.
+        """
+
+        @wraps(new_method)
+        def patched_method(self, *args, **kwargs):
+            old_method = getattr(cls, new_method.__name__)
+            old_signature = inspect.signature(old_method)
+            try:
+                old_bound_args = old_signature.bind(self, *args, **kwargs)
+            except TypeError:
+                # If old signature doesn't match then either:
+                # 1) new signature will match
+                # 2) or a TypeError will be raised showing the user information
+                # about the new signature.
+                return new_method(self, *args, **kwargs)
+
+            new_signature = inspect.signature(new_method)
+            try:
+                new_bound_args = new_signature.bind(self, *args, **kwargs)
+            except TypeError:
+                # Old signature matches but not the new one (because of
+                # previous try/except).
+                return old_method(self, *args, **kwargs)
+
+            # If both signatures match, decide on which method to call by
+            # inspecting the first arg (arg[0] = self).
+            assert old_bound_args.args[1] == new_bound_args.args[1]
+            if hasattr(
+                old_bound_args.args[1], "context"
+            ):  # Looks like a response object => old method.
+                return old_method(self, *args, **kwargs)
+            elif isinstance(old_bound_args.args[1], HttpResponseBase):
+                raise ValueError(
+                    f"{old_method.__name__}() is only usable on responses fetched "
+                    f"using the Django test Client."
+                )
+            else:
+                return new_method(self, *args, **kwargs)
+
+        return patched_method
 
 
 class SimpleTestCase(unittest.TestCase):
@@ -555,171 +675,119 @@ class SimpleTestCase(unittest.TestCase):
             real_count, 0, msg_prefix + "Response should not contain %s" % text_repr
         )
 
-    def assertFormError(self, response, form, field, errors, msg_prefix=""):
+    def _check_test_client_response(self, response, attribute, method_name):
         """
-        Assert that a form used to render the response has a specific field
-        error.
+        Raise a ValueError if the given response doesn't have the required
+        attribute.
         """
+        if not hasattr(response, attribute):
+            raise ValueError(
+                f"{method_name}() is only usable on responses fetched using "
+                "the Django test Client."
+            )
+
+    def _assert_form_error(self, form, field, errors, msg_prefix, form_repr):
+        if not form.is_bound:
+            self.fail(
+                f"{msg_prefix}The {form_repr} is not bound, it will never have any "
+                f"errors."
+            )
+
+        if field is not None and field not in form.fields:
+            self.fail(
+                f"{msg_prefix}The {form_repr} does not contain the field {field!r}."
+            )
+        if field is None:
+            field_errors = form.non_field_errors()
+            failure_message = f"The non-field errors of {form_repr} don't match."
+        else:
+            field_errors = form.errors.get(field, [])
+            failure_message = (
+                f"The errors of field {field!r} on {form_repr} don't match."
+            )
+
+        self.assertEqual(field_errors, errors, msg_prefix + failure_message)
+
+    # RemovedInDjango50Warning: When the deprecation ends, remove the
+    # decorator.
+    @_AssertFormErrorDeprecationHelper.patch_signature
+    def assertFormError(self, form, field, errors, msg_prefix=""):
+        """
+        Assert that a field named "field" on the given form object has specific
+        errors.
+
+        errors can be either a single error message or a list of errors
+        messages. Using errors=[] test that the field has no errors.
+
+        You can pass field=None to check the form's non-field errors.
+        """
+        if errors is None:
+            warnings.warn(
+                "Passing errors=None to assertFormError() is deprecated, use "
+                "errors=[] instead.",
+                RemovedInDjango50Warning,
+                stacklevel=2,
+            )
+            errors = []
+
         if msg_prefix:
             msg_prefix += ": "
-
-        # Put context(s) into a list to simplify processing.
-        contexts = to_list(response.context)
-        if not contexts:
-            self.fail(
-                msg_prefix + "Response did not use any contexts to render the response"
-            )
-
-        # Put error(s) into a list to simplify processing.
         errors = to_list(errors)
+        self._assert_form_error(form, field, errors, msg_prefix, f"form {form!r}")
 
-        # Search all contexts for the error.
-        found_form = False
-        for i, context in enumerate(contexts):
-            if form not in context:
-                continue
-            found_form = True
-            for err in errors:
-                if field:
-                    if field in context[form].errors:
-                        field_errors = context[form].errors[field]
-                        self.assertTrue(
-                            err in field_errors,
-                            msg_prefix + "The field '%s' on form '%s' in"
-                            " context %d does not contain the error '%s'"
-                            " (actual errors: %s)"
-                            % (field, form, i, err, repr(field_errors)),
-                        )
-                    elif field in context[form].fields:
-                        self.fail(
-                            msg_prefix
-                            + (
-                                "The field '%s' on form '%s' in context %d contains no "
-                                "errors"
-                            )
-                            % (field, form, i)
-                        )
-                    else:
-                        self.fail(
-                            msg_prefix
-                            + (
-                                "The form '%s' in context %d does not contain the "
-                                "field '%s'"
-                            )
-                            % (form, i, field)
-                        )
-                else:
-                    non_field_errors = context[form].non_field_errors()
-                    self.assertTrue(
-                        err in non_field_errors,
-                        msg_prefix + "The form '%s' in context %d does not"
-                        " contain the non-field error '%s'"
-                        " (actual errors: %s)"
-                        % (form, i, err, non_field_errors or "none"),
-                    )
-        if not found_form:
-            self.fail(
-                msg_prefix + "The form '%s' was not used to render the response" % form
+    # RemovedInDjango50Warning: When the deprecation ends, remove the
+    # decorator.
+    @_AssertFormErrorDeprecationHelper.patch_signature
+    def assertFormsetError(self, formset, form_index, field, errors, msg_prefix=""):
+        """
+        Similar to assertFormError() but for formsets.
+
+        Use form_index=None to check the formset's non-form errors (in that
+        case, you must also use field=None).
+        Otherwise use an integer to check the formset's n-th form for errors.
+
+        Other parameters are the same as assertFormError().
+        """
+        if errors is None:
+            warnings.warn(
+                "Passing errors=None to assertFormsetError() is deprecated, "
+                "use errors=[] instead.",
+                RemovedInDjango50Warning,
+                stacklevel=2,
             )
+            errors = []
 
-    def assertFormsetError(
-        self, response, formset, form_index, field, errors, msg_prefix=""
-    ):
-        """
-        Assert that a formset used to render the response has a specific error.
+        if form_index is None and field is not None:
+            raise ValueError("You must use field=None with form_index=None.")
 
-        For field errors, specify the ``form_index`` and the ``field``.
-        For non-field errors, specify the ``form_index`` and the ``field`` as
-        None.
-        For non-form errors, specify ``form_index`` as None and the ``field``
-        as None.
-        """
-        # Add punctuation to msg_prefix
         if msg_prefix:
             msg_prefix += ": "
-
-        # Put context(s) into a list to simplify processing.
-        contexts = to_list(response.context)
-        if not contexts:
-            self.fail(
-                msg_prefix + "Response did not use any contexts to "
-                "render the response"
-            )
-
-        # Put error(s) into a list to simplify processing.
         errors = to_list(errors)
 
-        # Search all contexts for the error.
-        found_formset = False
-        for i, context in enumerate(contexts):
-            if formset not in context or not hasattr(context[formset], "forms"):
-                continue
-            found_formset = True
-            for err in errors:
-                if field is not None:
-                    if field in context[formset].forms[form_index].errors:
-                        field_errors = context[formset].forms[form_index].errors[field]
-                        self.assertTrue(
-                            err in field_errors,
-                            msg_prefix + "The field '%s' on formset '%s', "
-                            "form %d in context %d does not contain the "
-                            "error '%s' (actual errors: %s)"
-                            % (field, formset, form_index, i, err, repr(field_errors)),
-                        )
-                    elif field in context[formset].forms[form_index].fields:
-                        self.fail(
-                            msg_prefix
-                            + (
-                                "The field '%s' on formset '%s', form %d in context "
-                                "%d contains no errors"
-                            )
-                            % (field, formset, form_index, i)
-                        )
-                    else:
-                        self.fail(
-                            msg_prefix
-                            + (
-                                "The formset '%s', form %d in context %d does not "
-                                "contain the field '%s'"
-                            )
-                            % (formset, form_index, i, field)
-                        )
-                elif form_index is not None:
-                    non_field_errors = (
-                        context[formset].forms[form_index].non_field_errors()
-                    )
-                    self.assertFalse(
-                        not non_field_errors,
-                        msg_prefix + "The formset '%s', form %d in context %d "
-                        "does not contain any non-field errors."
-                        % (formset, form_index, i),
-                    )
-                    self.assertTrue(
-                        err in non_field_errors,
-                        msg_prefix + "The formset '%s', form %d in context %d "
-                        "does not contain the non-field error '%s' (actual errors: %s)"
-                        % (formset, form_index, i, err, repr(non_field_errors)),
-                    )
-                else:
-                    non_form_errors = context[formset].non_form_errors()
-                    self.assertFalse(
-                        not non_form_errors,
-                        msg_prefix + "The formset '%s' in context %d does not "
-                        "contain any non-form errors." % (formset, i),
-                    )
-                    self.assertTrue(
-                        err in non_form_errors,
-                        msg_prefix + "The formset '%s' in context %d does not "
-                        "contain the non-form error '%s' (actual errors: %s)"
-                        % (formset, i, err, repr(non_form_errors)),
-                    )
-        if not found_formset:
+        if not formset.is_bound:
             self.fail(
-                msg_prefix
-                + "The formset '%s' was not used to render the response" % formset
+                f"{msg_prefix}The formset {formset!r} is not bound, it will never have "
+                f"any errors."
+            )
+        if form_index is not None and form_index >= formset.total_form_count():
+            form_count = formset.total_form_count()
+            form_or_forms = "forms" if form_count > 1 else "form"
+            self.fail(
+                f"{msg_prefix}The formset {formset!r} only has {form_count} "
+                f"{form_or_forms}."
+            )
+        if form_index is not None:
+            form_repr = f"form {form_index} of formset {formset!r}"
+            self._assert_form_error(
+                formset.forms[form_index], field, errors, msg_prefix, form_repr
+            )
+        else:
+            failure_message = f"The non-form errors of formset {formset!r} don't match."
+            self.assertEqual(
+                formset.non_form_errors(), errors, msg_prefix + failure_message
             )
 
-    def _assert_template_used(self, response, template_name, msg_prefix):
+    def _get_template_used(self, response, template_name, msg_prefix, method_name):
 
         if response is None and template_name is None:
             raise TypeError("response and/or template_name argument must be provided")
@@ -727,15 +795,8 @@ class SimpleTestCase(unittest.TestCase):
         if msg_prefix:
             msg_prefix += ": "
 
-        if (
-            template_name is not None
-            and response is not None
-            and not hasattr(response, "templates")
-        ):
-            raise ValueError(
-                "assertTemplateUsed() and assertTemplateNotUsed() are only "
-                "usable on responses fetched using the Django test Client."
-            )
+        if template_name is not None and response is not None:
+            self._check_test_client_response(response, "templates", method_name)
 
         if not hasattr(response, "templates") or (response is None and template_name):
             if response:
@@ -747,21 +808,7 @@ class SimpleTestCase(unittest.TestCase):
         template_names = [t.name for t in response.templates if t.name is not None]
         return None, template_names, msg_prefix
 
-    def assertTemplateUsed(
-        self, response=None, template_name=None, msg_prefix="", count=None
-    ):
-        """
-        Assert that the template with the provided name was used in rendering
-        the response. Also usable as context manager.
-        """
-        context_mgr_template, template_names, msg_prefix = self._assert_template_used(
-            response, template_name, msg_prefix
-        )
-
-        if context_mgr_template:
-            # Use assertTemplateUsed as context manager.
-            return _AssertTemplateUsedContext(self, context_mgr_template)
-
+    def _assert_template_used(self, template_name, template_names, msg_prefix, count):
         if not template_names:
             self.fail(msg_prefix + "No templates used to render the response")
         self.assertTrue(
@@ -780,17 +827,41 @@ class SimpleTestCase(unittest.TestCase):
                 % (template_name, count, template_names.count(template_name)),
             )
 
+    def assertTemplateUsed(
+        self, response=None, template_name=None, msg_prefix="", count=None
+    ):
+        """
+        Assert that the template with the provided name was used in rendering
+        the response. Also usable as context manager.
+        """
+        context_mgr_template, template_names, msg_prefix = self._get_template_used(
+            response,
+            template_name,
+            msg_prefix,
+            "assertTemplateUsed",
+        )
+        if context_mgr_template:
+            # Use assertTemplateUsed as context manager.
+            return _AssertTemplateUsedContext(
+                self, context_mgr_template, msg_prefix, count
+            )
+
+        self._assert_template_used(template_name, template_names, msg_prefix, count)
+
     def assertTemplateNotUsed(self, response=None, template_name=None, msg_prefix=""):
         """
         Assert that the template with the provided name was NOT used in
         rendering the response. Also usable as context manager.
         """
-        context_mgr_template, template_names, msg_prefix = self._assert_template_used(
-            response, template_name, msg_prefix
+        context_mgr_template, template_names, msg_prefix = self._get_template_used(
+            response,
+            template_name,
+            msg_prefix,
+            "assertTemplateNotUsed",
         )
         if context_mgr_template:
             # Use assertTemplateNotUsed as context manager.
-            return _AssertTemplateNotUsedContext(self, context_mgr_template)
+            return _AssertTemplateNotUsedContext(self, context_mgr_template, msg_prefix)
 
         self.assertFalse(
             template_name in template_names,
@@ -1205,7 +1276,7 @@ class TransactionTestCase(SimpleTestCase):
                 # tests (e.g., losing a timezone setting causing objects to be
                 # created with the wrong time). To make sure this doesn't
                 # happen, get a clean connection at the start of every test.
-                for conn in connections.all():
+                for conn in connections.all(initialized_only=True):
                     conn.close()
         finally:
             if self.available_apps is not None:
@@ -1242,25 +1313,6 @@ class TransactionTestCase(SimpleTestCase):
 
     def assertQuerysetEqual(self, qs, values, transform=None, ordered=True, msg=None):
         values = list(values)
-        # RemovedInDjango41Warning.
-        if transform is None:
-            if (
-                values
-                and isinstance(values[0], str)
-                and qs
-                and not isinstance(qs[0], str)
-            ):
-                # Transform qs using repr() if the first element of values is a
-                # string and the first element of qs is not (which would be the
-                # case if qs is a flattened values_list).
-                warnings.warn(
-                    "In Django 4.1, repr() will not be called automatically "
-                    "on a queryset when compared to string values. Set an "
-                    "explicit 'transform' to silence this warning.",
-                    category=RemovedInDjango41Warning,
-                    stacklevel=2,
-                )
-                transform = repr
         items = qs
         if transform is not None:
             items = map(transform, items)
@@ -1329,23 +1381,7 @@ class TestData:
         if instance is None:
             return self.data
         memo = self.get_memo(instance)
-        try:
-            data = deepcopy(self.data, memo)
-        except TypeError:
-            # RemovedInDjango41Warning.
-            msg = (
-                "Assigning objects which don't support copy.deepcopy() during "
-                "setUpTestData() is deprecated. Either assign the %s "
-                "attribute during setUpClass() or setUp(), or add support for "
-                "deepcopy() to %s.%s.%s."
-            ) % (
-                self.name,
-                owner.__module__,
-                owner.__qualname__,
-                self.name,
-            )
-            warnings.warn(msg, category=RemovedInDjango41Warning, stacklevel=2)
-            data = self.data
+        data = deepcopy(self.data, memo)
         setattr(instance, self.name, data)
         return data
 
@@ -1372,8 +1408,10 @@ class TestCase(TransactionTestCase):
         """Open atomic blocks for multiple databases."""
         atomics = {}
         for db_name in cls._databases_names():
-            atomics[db_name] = transaction.atomic(using=db_name)
-            atomics[db_name].__enter__()
+            atomic = transaction.atomic(using=db_name)
+            atomic._from_testcase = True
+            atomic.__enter__()
+            atomics[db_name] = atomic
         return atomics
 
     @classmethod
@@ -1392,42 +1430,34 @@ class TestCase(TransactionTestCase):
         super().setUpClass()
         if not cls._databases_support_transactions():
             return
-        # Disable the durability check to allow testing durable atomic blocks
-        # in a transaction for performance reasons.
-        transaction.Atomic._ensure_durability = False
-        try:
-            cls.cls_atomics = cls._enter_atomics()
+        cls.cls_atomics = cls._enter_atomics()
 
-            if cls.fixtures:
-                for db_name in cls._databases_names(include_mirrors=False):
-                    try:
-                        call_command(
-                            "loaddata",
-                            *cls.fixtures,
-                            **{"verbosity": 0, "database": db_name},
-                        )
-                    except Exception:
-                        cls._rollback_atomics(cls.cls_atomics)
-                        raise
-            pre_attrs = cls.__dict__.copy()
-            try:
-                cls.setUpTestData()
-            except Exception:
-                cls._rollback_atomics(cls.cls_atomics)
-                raise
-            for name, value in cls.__dict__.items():
-                if value is not pre_attrs.get(name):
-                    setattr(cls, name, TestData(name, value))
+        if cls.fixtures:
+            for db_name in cls._databases_names(include_mirrors=False):
+                try:
+                    call_command(
+                        "loaddata",
+                        *cls.fixtures,
+                        **{"verbosity": 0, "database": db_name},
+                    )
+                except Exception:
+                    cls._rollback_atomics(cls.cls_atomics)
+                    raise
+        pre_attrs = cls.__dict__.copy()
+        try:
+            cls.setUpTestData()
         except Exception:
-            transaction.Atomic._ensure_durability = True
+            cls._rollback_atomics(cls.cls_atomics)
             raise
+        for name, value in cls.__dict__.items():
+            if value is not pre_attrs.get(name):
+                setattr(cls, name, TestData(name, value))
 
     @classmethod
     def tearDownClass(cls):
-        transaction.Atomic._ensure_durability = True
         if cls._databases_support_transactions():
             cls._rollback_atomics(cls.cls_atomics)
-            for conn in connections.all():
+            for conn in connections.all(initialized_only=True):
                 conn.close()
         super().tearDownClass()
 
@@ -1786,7 +1816,11 @@ class LiveServerTestCase(TransactionTestCase):
             ALLOWED_HOSTS={"append": cls.allowed_host},
         )
         cls._live_server_modified_settings.enable()
+        cls.addClassCleanup(cls._live_server_modified_settings.disable)
+        cls._start_server_thread()
 
+    @classmethod
+    def _start_server_thread(cls):
         connections_override = cls._make_connections_override()
         for conn in connections_override.values():
             # Explicitly enable thread-shareability for this connection.
@@ -1795,13 +1829,11 @@ class LiveServerTestCase(TransactionTestCase):
         cls.server_thread = cls._create_server_thread(connections_override)
         cls.server_thread.daemon = True
         cls.server_thread.start()
+        cls.addClassCleanup(cls._terminate_thread)
 
         # Wait for the live server to be ready
         cls.server_thread.is_ready.wait()
         if cls.server_thread.error:
-            # Clean up behind ourselves, since tearDownClass won't get called in
-            # case of errors.
-            cls._tearDownClassInternal()
             raise cls.server_thread.error
 
     @classmethod
@@ -1814,20 +1846,13 @@ class LiveServerTestCase(TransactionTestCase):
         )
 
     @classmethod
-    def _tearDownClassInternal(cls):
+    def _terminate_thread(cls):
         # Terminate the live server's thread.
         cls.server_thread.terminate()
         # Restore shared connections' non-shareability.
         for conn in cls.server_thread.connections_override.values():
             conn.dec_thread_sharing()
 
-        cls._live_server_modified_settings.disable()
-        super().tearDownClass()
-
-    @classmethod
-    def tearDownClass(cls):
-        cls._tearDownClassInternal()
-
 
 class SerializeMixin:
     """
